# certView utility to parse and view X.509v.3 Certificates
# 
# Copyright 2019 Vladimir Orlov 
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#

switch -- $::tcl_platform(platform) {
    "windows"        { 
	encoding system utf-8
    }
    "unix" - default { 
	encoding system utf-8
    }
}

#encoding dirs <каталог с кодировками /usr/share/tcl8.6/encoding>
set mydir [file dirname [info script]]

package require Tk
package require msgcat
package require pki
package require textutil

set ::pki::oids(1.2.643.100.1)  "OGRN"
set ::pki::oids(1.2.643.100.5)  "OGRNIP"
set ::pki::oids(1.2.643.3.131.1.1) "INN"
set ::pki::oids(1.2.643.100.3) "SNILS"
set ::pki::oids(1.2.643.2.2.3) "GOST R 34.10-2001 with GOST R 34.11-94"
set ::pki::oids(1.2.643.2.2.19) "GOST R 34.10-2001"
set ::pki::oids(1.2.643.7.1.1.1.1) "GOST R 34.10-2012-256"
set ::pki::oids(1.2.643.7.1.1.1.2) "GOST R 34.10-2012-512"
set ::pki::oids(1.2.643.7.1.1.3.2) "GOST R 34.10-2012-256 with GOSTR 34.11-2012-256"
set ::pki::oids(1.2.643.7.1.1.3.3) "GOST R 34.10-2012-512 with GOSTR 34.11-2012-512"
set ::pki::oids(1.2.643.100.113.1) "KC1 Class Sign Tool"
set ::pki::oids(1.2.643.100.113.2) "KC2 Class Sign Tool"

array set ::payoid {
1.2.643.6.3.1.2.2 "МЭТС"
1.2.643.6.7 "B2B и B2G"
1.2.643.6.15 "Фабрикант"
1.2.643.6.14 "Центр реализации"
1.2.643.100.2.1 "Росреестр"
1.2.643.3.8.100.1.113 "Росреестр"
1.2.643.2.2.34.25 "Росреестр"
1.2.643.2.2.34.26 "Росреестр"
}

package require base64
package require http 
global loadfile
set ::chaincert ""
set ::listcert {}
set ::crlfile ""
set loadfile ""
global count
set count 0

::msgcat::mclocale en

set lang "ru"
set lp [llength $argv]
if { $lp == 2 } {
    set lang [lindex $argv 0]
    if { ($lang != "ru") && ($lang != "en") } {
	exit
    }
    ::msgcat::mclocale $lang
    if {[llength $argv] == 2} {
	set loadfile [lindex $argv 1]
    }
} elseif { $lp == 1} {
	set loadfile [lindex $argv 0]
}

namespace import ::msgcat::mc

array set dn_fields {
        C "Country" ST "State" L "Locality" STREET "Adress" TITLE "Title"
        O "Organization" OU "Organizational Unit"
        CN "Common Name" SN "Surname" GN "Given Name" INN "INN" OGRN "OGRN" OGRNIP "OGRNIP" SNILS "SNILS" EMAIL "Email Address"
    }

::msgcat::mclocale $lang
::msgcat::mcpreferences

::msgcat::mcload [file join [file dirname [info script]] $mydir]
font configure TkDefaultFont -size 10
option add *Dialog.msg.wrapLength 6i
option add *Dialog.dtl.wrapLength 6i
global home
set home ""
global typesys
set typesys [tk windowingsystem]

switch $typesys {
    win32        { 
	set home $::env(USERPROFILE) 
    }
    x11 { 
	set home $::env(HOME) 
	catch {tk_getOpenFile foo bar}
	set ::tk::dialog::file::showHiddenVar 0
	set ::tk::dialog::file::showHiddenBtn 1
    }
    classic - aqua { 
	set home $::env(HOME) 
	encoding dirs $mydir
    }
}

image create photo butborder -data {
iVBORw0KGgoAAAANSUhEUgAAAIcAAAAiCAYAAAB1NspXAAAACXBIWXMAAA7EAAAO
xAGVKw4bAAAAB3RJTUUH4wIMDAsSdUV+swAAAB1pVFh0Q29tbWVudAAAAAAAQ3Jl
YXRlZCB3aXRoIEdJTVBkLmUHAAAA80lEQVR42u3VsUoDURRF0XMf3sh8QGxiEeyC
TaJ1wCJBvzWdRdAwKYVUQhpJaYwz3euneMW1Eiy0F2avTzhsOBYRatsmDoc3lVKE
fnN3TSbXGo0u7axtmzidPjSf31lKSe7OQj23272EmcLq+jkWi3tjEvy03W4icSX4
TSlFiRnwF+IAcYA4QBwgDhAHiAPEAeIAcYA4AOIAcYA4QBwgDhAHiAPEAeIAcQDE
AeIAcYA4QBz4J3G4u3LOK6bAt5zzyt1lTfMZ+/2rhsMLTac3Nhics07PrdePMZvd
yiJCdf0Ux+O7uq5jmZ6rqkrj8ZWWywf7AmxXRGaRW9gDAAAAAElFTkSuQmCC
}

ttk::style element create RoundedButton image \
        {butborder} \
        -border 3 -sticky nsew 
ttk::style layout My.TButton {
  RoundedButton -sticky nswe -border 1 -children {
    Button.focus -sticky nswe -children {
      Button.padding -sticky nswe -children {
        Button.label -sticky nswe 
        }}}}
ttk::style map My.TButton -background [list disabled #ffffff pressed #ffffff  active #93cee9]

image create photo up -data {
iVBORw0KGgoAAAANSUhEUgAAAAYAAAAICAIAAABVpBlvAAAACXBIWXMAAA7EAAAO
xAGVKw4bAAAAB3RJTUUH4wIKDRkw/Hh6bwAAABVJREFUCNdj/P//PwMqYGLAAPQX
AgDFpQMNg52XXAAAAABJRU5ErkJggg==
}

image create photo creator_small -data {
R0lGODlhPABOAOf/ABQUHBwTExYVGBgUHhkVEh8UEB0WGygVFxkaIRkbGRwbFSAaFSQYHiQZFxgcHhwbHh8bGiQdFCgcFB8gJx8hHyUgGiMgIx0iIyofGyQhICkgISMj
HDIgHjcgGyskGTAjGjEiJy0kJC8kIDUjHColKSomJSwmICUoJzsoGEEnHjspHUAnIzItGDgqJzsqIzYsHi4tMTEtLDQtJzUsMi8vKDguLy8yL08qIkssJEUtL0MvJEcu
JEIvKTkyLEguKT4xKjY0KDczMjU1LTM1OlExKE4zKT43MTs5LUo2JTc7Ljo6MjU6QT86Kkc3MUs3K1szJE81Pl8yLkE5RFQ3Iz87OjU9STs9OlM3M0M9Jz48QDk+QFo3
Jlw2Llo2NUY8P1g5L2E2OD1BSGE9NmM9MWA9PFpANGc8PWw9N0VHSlFERUlHRUNIUGdBME1GTnE/MlRHQlRGT0tKQ2NFMFxGQXNBRGxFPGpFQ2JJPHJGMXpCR09OUVJP
TYZHOXpKQlpSV3ZMQnlNPXJOTXBQRIFNP2RTYV1XVntROlZaV19YUXZTP2xVT1hZY1laXY5TWIlWUYtXSYhbQG9gYH9eUGpjX2NlYIhcTWtkWHZhWZFbSHFhbGBna4Rg
TGZmbIZfXGtlcZViVpRnWmxybn1ubJBpaJVqUXZve5JsXnJ0d4tvYXJ0fpVuWnt0bmx4gX97jKR1ZK10X595WKF5X5p7bXyChJ97ZaF6boWBfJx7d32Di3iEk5CAfLiD
eqyJbKmIhayKcrCIe5qOiYmSoa2Lfa+NaI2Sm46TlqWOiKiPfrmTdrKWj8WShLeWhr2UibyYgZqfqJyho5mhsaefnL+dj7Khlrmgj7qfmMaio6WstMGmn8elmcKnmM6k
l6qvsbKuqL6tpKmywciun8+toMqupsyxtcW4r7S8zNq1o9i2qda2sNK4qdG5sNe5pb6+wLvAyb3E1N/BrN/DtOPDvd7GvcPM28zMzM7P2crT5OzQx9DZ4dHc7dfk9vrf
0+Pq6eDt/ebz+////yH+EUNyZWF0ZWQgd2l0aCBHSU1QACH5BAEKAP8ALAAAAAA8AE4AAAj+ABMkeGBAAwYNGg5oCFEhw4YEDhxYsCBAAIUYMnrIMFGiB5AeIIGIBCKk
JJAjST4eYXLkCBAlQjYEMFDRQQIBDxJMzLAQoQYRIkqUoCAgwQaiFmnQiME0RpAeT0H2cFmyJUqVTE4eUQJkAwECRQVCTECBZwiEGESYCGEiwwMKZB9UpEBDho0YNIRI
lWoEphIlLZMItlqyZA8PCyoKzGkzAQSfbEWcDWHhAQQIFhDMZWqjZOe9RvRWdSlYsJCWhYUgBrt4oMDHFdZKRuj2gYMHJB4AGHCBRA3PnT2fTi1EpNWWI0/GXCBWbETH
FSqUCIGhAu0Hby2UsDBgAA0rVNT+xFFD3ooVJeb/CvkbePDxkRUgKBZr2/GGhhkyrA1R+ygFEmEwEsopnJwSSiicHFJIIQr6cUgcSfyVBEruWQXfV85h54ABD2zg4Ych
hLDdBRCccMgsxBRTDCuzcKLJKZowwsginHDCyCHnlaREacW9B4QMGmDYGgIIYEdWWfllUAIJF5wQiopQ4jKLJlRqssiVVCIYB3HoWTGaS0cUt8ECCogFAUQ1CbRTBg4g
MMEJszyjopyz4AKjJpQwYuUilIQyZRxKBCeEeVZsBRhyQtAwZpmvJaDZfBmciV1OhTxjKTHO4CIlJ6mkwkqVp5wyyyyhUGJFZyTtOCFMJ8GUKAT+BZzp2kDYFUkBBRbQ
moAN3FjqDKbESFlnnamcwsopxcLICBqpaVUcSIX9WMECZJZ5mQURIXACCdyS8GYCz3DTqzPkEhNsMcJy4omLVObJSI6F7XiEUj1EK0ME1SqgLwQTXOAvriW0VRkavT5z
zcHlFmNusLh0qgknlByihx7nCQqYEjIoJQNJxXmgr7WX9dvktjCcEJEAlhosbsHPkItuKlSuy8kie6iRhRdG3FWCaErUgFfGStGAQQMNVBBBdBVYQMEJMMDgFAkDWdEN
MSqLm3KwmqzBiaYw6vHGDzzw0EQTR2ikWgkYxXCXDTSYsEEHI2DwgQcfzH0BAhbYIJT+iBc4oEbLgFuq8KeL4JLiKX54UQPYYfOgww9GaKRRCQycIIMMJaBa1wsvqMD5
CyxYMMEEJCzJ7USHkNuyig1TiQu5pxQCRQ4+7ICEDjzQHrYOTfzgAkglXBYw2zS0ZAQWWLDUgxGiTzQRCRpYoAEjdK5iCyWr4FKMwcREMkcZZIgxRhFF7KBD7ebr4ETY
LnDQQgsVbCAUUxlEFwG+C0SAAd7OT4SdAWEoxixsYQtgFDAachJFIBJhCEAA4g91GAMRnFAEHeggfRZcwQpawIG0LEBJQpkWvgpAQglIgEhFmhR2JmAAZ7CjG92Ihgxh
CIxj/IIWqogFKCpRiU1Iggz+VyjfDpxwux1osH0HiZRQTGC0+y2AhBE4IQofwID+BQGG3SBHN6YxDXV4AxzpAAc1lnEMX8CCFKSoBAS5wIUobGEMW0ifC0bwgQZsoAEW
IEGk8reArxSgASd8AAIGAAABbGgCFKBEFsnBSG84UhvpCKMYj0ELUmxiE2o8wxnYcIYo1IENFdTBHD+AgQg0AAKRmtYCAkCAAASAaESSSwofYAEY2GKRjCQHNr4YyUhS
wxeq+IMchtmHYvahDmeogyC+IMoV0LGDp7yMKltJgD82YIoAwI4FsuAHXDayGmCMJDi0QY1jqCIRbBgDG/rgiEGocQx1qEMZ1qcCFXwAmpf+uQy1XElCWKIQAAgwQN6A
QQ51ZNGR0wCnOKmhjWWgQhJlKMITnsAGTJzRDTcoAhLWx4M5jmCO0clAH1cZgAK4kmgVSaEBDMAAIXgDG/JwpCPFUQ1xgGOc1KCGJGQhCydMYQtthMQwSPGEHaggBSlQ
Afta4AJSaiA+X6kmCV+JgYpkM6ArjYM2dhEPLzoyHTUdZzNoQQthiKMXlQCEHLjghlgggxRsQEEH5lo3FYhyjhwQgQeiGtVXolQAhSQIh1ogjVecw6beUEc6tGFTcAy1
FssQRi1AgQc24IEUzViGK86Ag7lyAJAjSIFH87rXvpZ0ARVoAGABkE2CNIADynj+xTbQIQ55qGOc6rjtWGlhRlVAwhCVcCszmKGKM9xgrh3gAAdGENpniqACJm1lAJ64
AAxwYDesFcBKD3CAXexiG9LQRkyxAUnFasMXvIAFJipBiljE4hfgFYYp3LCFFCR3BXOcK3NHIAIOsNIArixpAzAggQeARbsrZcABXKEMZTiiGjGdBiThIQ90LGMZlSSF
K14RC2ZsIxvSMMUZJGjfFaRgBR1wJh1dgIECrNSVCxhwKbGzmwGs1AAHAMU2PvEJadhWHJGUR4WlsYxfuNcVpMDEL7JhjnDUwg1nIMIOkEpl0Ta1BR/oJ9Hy14AFFDgz
3bkxA/6wDVfE4hjwSHP+muNBWxAzgxc6xAQmmiGNX0h2EH3gAg5wUGUc4NcFLRBBlytQHaNJIIp5PMEELMBSntwBvNKQBYWFbFt1ZOPSzPiFevkw5z5EQRby/QMRvrCD
Hex5zytAgQt+8NkIaCADHqiABzzwgg88byIaIAEISnAHIp8jHJOWBzwUCw6aMsMXrhjEIEzBC1LU4hagEMRRi+CDU/N5BSr4nQcAiQFae6DbHuBABiiwnW6BYAY5GO45
4nEPSsMjHukQB6Zr8QlHCMMXvuCpJARRhnlecAdFIAIOdjBH321bAuCuG61doCQS5JFb567BLbYB7ElPWh3iEEedQeEKMh6Dp7JAxR3+BHGHOdwOB0QQogtc0FEJDHjW
Clf4CfYWAojXoAaKiMc7gi1kjIsDG8swxS+kkYxeGOMYqECFIu6w9Dc4oXZFGLgPOvqDFmDg6hGY9f2+bYKZLylEJJhBDWbwA13Eu92U9rk2jiEMbRjj7Ue/hSkCcQem
f+8K1S711HlQ9aFdXW6z9oBaSjBzboUgBmKfARSacIlq3CPNPbepNpKRDGwYQxfGQIUpRhEIMZThCt+bQxOCKHDc8Z2DDfjAC1yQVBTUUwVCWVJTxE4FL3hBEbVVMzwk
+fNkVCPpx1gGM3rxCTdwgQg+yMHY8B51Uf6A1cql4wdG0IG51W1JM5gBU7L+XwMlNGEOl2D37tMBj1+DA+PTuIQstAEOIvfiEX0YMbVX8IPlO+5xPbB6WmZ9EKRVAAYk
0DQzkBFBUANG8H2o0G7lR360JQ65NQ2SIA3yFlmqYAhuMAZj8AVFgGI+4AROYEEqADkt4HJDQzRbFh0woH1OEQPL4wW1dwW3oIDpEA7kF0bxZgzCkA00hWGkMAhnwEZE
IGUAdwVNEDbPV3UuBwHUElX55DRqAxVBYARUAB6UMA0yGA7Adg6RlA3VUA2XVg33RgufkGdPEITl4wRl0Ds/IASRY3UNsQEKQABicSsx4DQ2EARBQAW1ZwWnwA7YEA9p
9mvhcA5aeFjogA7+4ZANmcYLrvAJgzAGxydwReCBfBcaR1B1IQWHzUGHToOHeXgzcUAP/KAOgLh7WEiIWIiFh3gOYMgLqgB/UVCGAucDPmBBP2CARmAEL4AB+VECcSiH
CXACJ1CHMDAEQ5AF4WEF3OAP/UAPpAgP63CKiSgN1KiD6FANRkYKj9BJbHR8OFA7IfgUiyMDDTERArEBJ2ADVjAENmCMQyAFWUAeocCM+lAPh3iIqRgORHZh1EhktWAK
oOCDkPgFszh1HgEEuWgEMoABZQEXG0ADd2gFNDAEVpAFUiAFthcG/NAP+tAP+TAO4oAOgjiNF7YM1ChZkzWGf5BOX4B3PrACPND+A34ROeSYH7eyNDZwhzGQBTyZBWEQ
BpxAj/owlO1QDdggb5cGYiZJjZEFTJUwCBCEgQTpkpCTh0qQiz3QFuMmEOloAzfXNGtQCq0ADfjgkfnQkf1gD7pQDdUIYvu4DM3QDMjglJUQQXAUhLTIAyyxHlepEeN2
K2TRlTBgBdtzDd/gDvPgDh5pD0PZD/3wDJewDNTQj8uAb8LAC/nmCqqACfHHBV9AQdQ2dVX5F0YQBPBjk0vzHTBgA0qAmNdQDvPwDeWQlvZwlo5pD4gQCMIgmZHFC8Pg
C7GgCpOlCpUQfxrYBHg5dbk4A30RJjXAiw5nAReQkzlpBe1QDu1gD/b+AA3foA/24A62yZHOcAWCUAvCIAy0kF6xYAo85AhqFEEaWARdkJdHkJDscQRAkkfO05VagAb1
8A3tkA/lAA3QoJ3giZZpSQhkcAadYAqqYAqbYAjx9AcPVAdiIAYaGER49wMt4Sp/8RT8ITq3IYxZoAdo4A7f8A3bSS7z8J21iZb64AwrAAZ2IAiS8Ad9wAZicAZ/EJUX
+gUt6QP29QE04KEf2gMi8B8lcwLGqAVhsAQo2g7uAA3OcA2IiZhn2ZjuAAVdIAbxNAZiwAVkUAeb8Afi8wUXWgY7QAajIHKKEAlvYChCEAQ/EAIwsARDoAVa0KR46g7g
+Q3k8g2x6Q7+5cCYjWkPkXAFXKBOkNgFXVAHiVAHXQCmX3AHl+AN8SAO1lANlHcMktAEf0EFNRAC/akFS3CqWaCnVuAOtUmgzqCiglqo+mCb3zAJV/AFiwqJXOClyEQG
XHAFqKAOtjUO1mANySBZpHAGoGoENwcCVqAHayAFYcCTqsqq20mg4BkM5aCiZ5kP0LAIVxBEX4CBbSQ+8SQGnYAN7aap1dALyXALnVAHX5ADoVqAIKAGa1AFVXCq/Aql
+YCiB4MP5ZALslmwucAKcBCuQDoGUcBGaCo+ydBuGIcN1jAKdqALi4CRU0gFxggDLUACWpCv/XqqQ1AOAoqighoMwZCiwQD+DQcbDAmrqIsaBWAApGJwB+cgZL3XC53Q
B5fQDhmLjFQQsm0gBSIAAkugryOLp/Pwr/Ygm+WgstAwoPMQDKzACl5wq+MqBlGAoVsgCeFwD2H0rpIQCHZABrowD4swrVOYBYvgCcFgAEirr/t6qnWbDy36tFOqKVM7
oGsAB1BwBY7KBemEgWOQCO8gttiADbdABnZwfFcADPWwBnrgBT3JCJ4ADQNQAklLt0mbBUlrD/Mwq4cZDKlgONuKbmDgqF2wumOQTGMgCOmwD+qADZwqCILgqb8aDfWw
CGjQkyXqB9CAAGrQBmFAt1Xwk2GwBgeaD6V7usQADVIQuF0guI7+agY7uknLQLuLWw2jIAio0AlmSwbR0A6cAAcTowZ6wAh6gAsX0AaM4Af5qq9hgAZrcKL54JHl4A6t
0AqpUAypAAWBSwYETMCa5KXUsA/woA1d2At/EAidcLZiQL7skAp+sAc1gwZ6MASpMAF6UAgyIr9VoKc/qZhpWbWtgCzFQAhQAAbhc6F2YAd1EAWmcA8KXE7GwLN1wAVm
ULNXQMGccME10wZo8Ls2QAkLMgmTUAh64JNZgAbz0A/+oA/f4L+hUgpQUMDxZAZmYAeB0Avy8HNdaJS2uwy1IAmS0JJkAAzs4AlCrAZpAAeWWwNpgAiIsMSFEAnsuwRa
kL+OGQz+pVAsq5AJBSzD8VQHknAP6EANY0zG1NCpsmAKYvDDpfAMQYzBNZMGakAFPfAGSlwIlGAJk4DEe4AGfpwPweAJpVAKkbAKokAGdxDD8SQJ7Fa7tjvGxgByqCAI
8vQFZJAGq7DEmFwzNlMDlKDEiGAJokwJzMwI+sCM7pALq5wJomA9osAGMYy78LAPioxxjdwLkSwJFloG41oHTYAIlDDMe+AFm1wDh6DE8KzEkTDKUuy8qxwJopDPtqAL
l+DFssvNtrWp50l5RScLmyAG1fsFZSAGOeAH6TzME6MFnRwJheAH8zwJkTDPlOAP/rCdpUAI1awLurDP/CwI4QDQGPeirqAACrXADJwayXUQruFDBj7wBg+tzllQA4Gw
03YwB60sz5PAjFFbCtS8CtZTQMCgC+KAdoeIDQDZCY3gCJ3QC7WQDL/wB+QMpF2QA548zIiAwYrTCY5wBmc7B6JwzErMjN8gzZlQCiNN0sDgDfdwD7mlDtZwC8XkCI6Q
B3RAB6CADr/wiOKD0OccCRhcCJjsB1LQAoEQfxNMBoGgCBg9CQEBADs=
}

image create photo ::img::ru_32x32  -data {
iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAACgElEQVR4nO1XMW7bQBCcuVsxlm0IYBpLfkJe4FRJmSrII5zSvYtU+UYekL+4SOHKQMrAAgIVMWyAoGTe
bgqS4lGmlciilSZbcA+n5c1wNLdHEg+DHXN9hT0Gtpq7anoDjeZMjo6O6L0nyT8ReWrEa7XIzOdz4+XlJff39ydJkgycc3zkxj7CAMDKwGKxuM2y7Jfc3Nx8Go/Hn5Mk
6RlvDRMzZFmWX11dvZXr6+s3JycnEJGdEQCAJEn2Li4uXgtJTxKVB3YWJDGbzcTtFLUj/hP45wRkPjdkmZr34UkLWFefWxO11/NckedmcnoKnJ0BIoRzZUGd6+J4g5gB
IZRZdT2B+LfV+hCIoiAEIFQdVV0LbDWrNsCb7lizBrgmokqEAEpXkWqpQj1WbRbru13IKngdNfBz9Kd4zVb/jRXYVWNsKVBL3WW+PiNW28WTXY7edJttGlIe02Zm2jwx
sfXbwDreJY5BUJh8kY/2Pjkz7xzAqAH0dUIaYLBGYlNADXMNONccMnxxz9EBnXdt8MYIy8vmyFFqCJRmy1UxLBYUN3TAyFftDysEenLh0mA1AYJKMCeEBw5IH8oPcIun
b6HXb4MtBWgEb11NwHcQiMCfysGigcVECKqBPwlxhw58KdFTxwTaPP6OiXUOW4cBCKiBw0gBQ9/SdxBb+oAgDNijybfcfrxLfdWROuTvlUCVYcgLC99vbcp0dPgq3cMH
8UzoHAjQ1UdhpcZWFqguZrb8KFFVZPeYKdxXTiYTknRmxtFoxOPjYwwGA3jv6b3fuhmpKkIICCFYlmWYTqfI87z6LxBaeqdp2titAt6GgFXGs+hAKYrC7u7uliXrVn+W
b8PV+A25Z0BSvbyERAAAAABJRU5ErkJggg==
}
image create photo ::img::uk_32x32  -data {
iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAAHEUlEQVRYhcWXa4xV1RXHf/ucfc59z3CZFyMyZUSaltJBQBTqowWTWpGnlKiAtbVpK5QE0kYNaVK/kGhs
tKapwgc1LbYliNaxTRMaCVRDIwgUBq0MA1VaYHjMy7nv89q7H86dO/cy2JImDSs5Oa+91/rvtf5r7bWF1pprKbJ8N66BbVUNQF0DAADIgYEB3EBe57i+jQCBgMs8orUm
HrNpSMcJSi5+/8BYTRpkQxqZiDE4VCBXcBBCVI9QI7oEkEhEcoGT7Rc33rJ64zln8pMTmursqG2GQ4Wgeq5T9Fh81xd45vGFDB46xj/Xb0KYZq19x+X6n/2U5gW3sfmF
3fzmrS4iMauyAEaoJiCXdxnOljKN7l9Xy1VL5z6ox3153PunBrkwmCcZt7FMUR4b3gslj5aJrVimRbqhgVLbDWCKUNsIANdlfGMT0pA0XTeBhvYhEtEqAAL8QJMrerS0
20xvb4yf3PfxPfLm6RP9xUu+ykefDPDL14+yY88JhrIOyZiFNMNIZAsuM6c2hX4slih92A2mQbWbdMkhyOUA6L2U5XD3ReoSNgC+r8gVXeoSEe77+jQ2PnAzHYWLPNp5
Buno0Mi09gZefOwuNt4/i62/P8rOvT0MZkokYhZRy0AaVTGxJRi1AAiCyrtpiHCOCfmiR13M5qFFs1m3chZfcgcY2rqFj7btZPDMB0o6Pf+oieXn29I8t3E+a++7ia1v
HuX1PT30DeRR6rJEqY1AzbNSmnzOoT5h88iymaxdMZNp7gCDW7fQ/cYu/PP9iLjEamlBnnvyGT7Zf5ym9d8hOf8rFSVT29I8u2E+jy6fwXO/PUg8Iiv/NJpRVlV/CyVq
GXx76Qwe+9atTHMHGdjyAt2db+Nd7MNIxKlbvIDG768mvePXSCMaoXDgCGeOdRO/Yw6N31tF4tZZoYuBqW3j2bLpbrJ5t8qaDq+alY8C2PDgHFKDlxj61cv0vPU2/nAW
EbGpW7iAhu8+QPLOueHAHduQ12/exNSF9wICVSwibJsrSapMKC2qPKA1KI1WCuU46HKYUgkbP2MRW7GIyQ+tDLkhBHbbxFqlWiPtyZOwJ0+6otEriZAS5TpoNFrrilFV
LIIcrV+ytQXZ2vJfdJnI3s0/5/T2XWFhEQIMAcIgrIqAEKGhIECrgCCTJfC8MglHmWdEI5x/6hfI8enR8JRXGb6G3hoJnwCGD+9DFo58QKbnPFhWWAFNo5xi5fkqQPt+
WEzQCMNERCNjl2MYFP52DB0E1KTECACtw1TVGpRCaHCKfUgRiSDiMYQMPSCMEIDWCh34YEqEJWsy7jNdGomMHVcNQKkaAMKXhJq1RisdbkFaof0Arct5L67G9P8ucgQl
WoESKKXLrP2/2h0FoB2HQOfBFGgIt1BzlAOI6p35P3AA0I5Ty4Eyb7QWofsvC4H2fGR8Zgf1bVPQ5VULYYSZMCKi9jnIZCkc/XBsaJQiPqujNgvK9woHqMoCDZFD+5Ct
T6ynfcnSq3ZZ6fhJTixcE5K1CoRyS7Ru2khy3uyr1lX/4x8htReM+eGe6UUHobtUoYSsS2JNag0NeQHCiofpWgVABGY45zNEK4Vz+iyqUMKMxxCGwO8fQla3TbkDXfS9
8gb5/V0ERQczGaP5nttpfmQFwzmH+mQk5ISdHNMPCGWBCLukTN6t9AIQ7hICQTCU4dJLO8nufR+8gE8/PYkUpknhcDeXXn6T4XcOEWTzWI3jmPDN+bQ8vIQTqWZ+sr2L
hDzL8xtuRxgmRMYC0NoGIwTw7PajnDqb4Yk1M+m4saHcZQoSs6fTPns6ufe66H/lD9B5Gtn7/A4+LiTwBoaRjQ1MWPkNWtbcTU+qiaf/eILX/nyQ3otZfrBsWnmpJiJS
N4YDQrsII8xqVxv8bs9pdh/rY/ntn2Pd0i/SMSVdGZucN4PkvBmkg7PI4skLMGU2Ex++k+ZVX6Mn1cjTfzrFzt1HGMw6JGM2ifHjMEd2ScOESD3aEJdlgocuAzAtm3i6
Hm1JXv3LWToPXGDpbZNYt2gqM24YV5kh65PIlkV3yOmPr6UnleapXad5bc+7DGVdkrEI9elo2b8+woqU7UuMigdGzSt8TDMEKSwbInHMqKQuCoFSvPruBToP9rNs7kR+
eG87HZNTWKaU8kC9+lfXe8O3bN/7dwazLsmoJFUXq5AnhAqu0ig0mZJPTibKWVDNcp+MExAFHF+DKdGmZKR4pVIWgdJse+ccnQf7WD4nzfEjx84JKxK/ybNb7jdtO25L
A2EYZc+KysFCaUgnJJMao3gFD/dihjGiNVZzHXYqQu9Aif6sV6lnWuuay/OVEXhOnyz0viQO7N+PDlw8zzMM08AQomJ8BIAo9/ReoMPv1pWPktpXaKWxTIE0RVVBVJWe
QClFECjiiYTKFx3EtT4dX4tTcY38G9XkGJmEnXNDAAAAAElFTkSuQmCC
}
image create photo ::img::us_32x32  -data {
iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAAHDElEQVRYhcWXfXAU9RnHP7+9zeXucgdJLuRIwuUFmoCUYIWEl4BCm4CNCBVMywRqZyyjHVERplTaTrFO
qWUmo9NSpR2ndVpbbW1HO60vFWrVAj0FK5ggBkiwgQTSvF28l9ze7d7t/vrHHSGhHVFpJ8/M7rPP7s1+n9/v+d7zfVZIKZlIUzNemQBsa2wC1gQkAIDa1z+AJbKKk0bK
bloSKSU2RVGEIrAsC0tKFCFQFCUdWxJFGROPeS4tSY7LTn5uDpb1H6Udt0hFUUb6+vqGxMZNW7ce6lC+K6Swz6/2s3p5NXueOkB0JEFVRSEbVtXw2G8OMDQco7Qkn03r
FvH47wP0DkTwed1sbr6eJ//0Fl3nh7FSKdbdeB0PfX0tWtz40JULiGzfvn2jeuzUQHPW5Fm5c2aXgU3hzLCBr6KMUtWGqip0DuoU+P0UlStI4MygTl5xMd6SElKWpHNQ
J6fQx5wCHwOhEXylfux2J3a780q777K7nI2CSTcEGjduqLuzaTFf2fUc0ZDGstoZfOPLS7hz9/P0DoSpmT2NXV+r595HXuJMT5DZFVP44X2N3L93P22d/ZQV5fLT+1fz
7ScOMCs8TMt0FzriEpTMnCQg094yEjx44I8Pi6LqdQH7tOvqfIW5hCIa00vyeL9nGJ/XzVAoRqXfy+lzQ0wt8NAfjFJVVkDHuSGmej30BaNUlRbQ2RPEl++mI6ixvPuf
fPPMu2iKbUwCMpODBMtCSEFSC/OwdbZFWb6wkrCW4K3j3VQUTWbzugVocZ3A8W58eTlsvnUBUkoCbd1Mctm559aFOOw2Am3d2BS4u2kh+R4Hb77bQySi4SgqwNVQi7u+
BvfnMsfodS3u+gW462vxrKwju6QYYfffGLh26Yq6hkVVPPnSO5imRXlRLo2Lq3hqXytxPYkv38Mty67hmf3HiWg6uR4H6xuqefb19wiGNHIcdjZ8/lp+u7+NlYur2LNj
3ZXqD8C9W7a0KEbcIMum4J3sIpVMEUsY2BRBQa6LVMoknkgikBTkujAtCy1hIC0Lb24OwpJoCQPLMpmS50K1CaTy0XuaUG2INRu3BQI9zrpgOEHDghnc/oUaduz5M/3D
MRbPmcY9zUv4zmP7OdcXovpTPnbc/lm+/7PXOH1ukOnFeTx410oe+eXrtJ7uxebMpilXsNMn0aUAkSaiEAIQCNL3hBRYSZ1dh19uUc9eCGK3lVLpz6dvKMy+wClURTCz
1MtwSGPf308hLcnMsgJimsHLh9qJjWhU+DzEIiM8/8o7DA2GKCt0869YimhHF8G/tKIpKqM6c9FbEhAoQoBpEqcXW39o8qbrb1jo33lHPb968RhHWs9RO8fP9zav4Nm/
HudvR7uYVe5l95ZGXni1jVcPtFOY7+bH31rLwX+c4ZWD7Xg8TvY+8CWOnbqAV5qsmj+drJkVuGZW4Kwqxznqy3FUleGsLMVZVc5hIxwQFbXrA4ansm5SjgOJZM6MqRxt
P88kdzaGYTK3ciqHW7twu+xocYN5n/Zz9EQPblc2kViC2upSWtvP43TaOXs+yG1rF/GTB5o/Ege2bL2vRamdW85ILMHJjl6KvB6aVsxFi+u0nbyAx6HStKIaM2XxXkcf
Toed5ptqyMqycaKjF0URNN9Ug8ft4ERnH7F4EkV8HGEVCFvxysD8pfV1q5bN5tGnD5EwUswsLWBtQzV7nz7EiKbjL8rjtjW1PP67NwiGYhR63dzxxcX84g9H6B2IkDvJ
yV3NS3nixaPMiwTZVSjRMxCjkiTluENI+MHxgy2qGTcIR+MMh2OEo3GSCYNQxEkoohGN6YzEDaIxnVBUIxqLE9V0nA6VDyJxorEEUU3HZhOEIhojeorw+z2E3zhJLNMJ
xxHRtJASBCYCSYJBRO3KTYHi6fPqkikTV7aKEGmy6noSl9OOEALLkiT0JE6HHSHSL40nkjgcWWlGS9ASBsKRzcJklCatH310xpGXtMBKt2QhJUJKHnr7tRZ15903s3rN
LR+jbv87m7JtG4phTtgwhDRN1K7dj9L26xcyXSvTvTIdLK2oIu0vsumiyv63+PJnYwM5Vo4lAhhqexM12dFOovc8UggQSvotyhjgsbr+ydY5bg4ACywLG2CGB1DzGpbg
m30NkjErHwW/2gTkZZdyNBFFkeTs249asumrlDfefBUgn9zytDiq1OOXxGJ88f6PltnVpIEafuZH6sCJ5zBNLivDpd+NLYL8kPhKPBzbE7KzQD96TFXf7jzZvXRKcIGC
kk5gFPFqyXe5XQJXgJFUilPdFy4Ij8f9mVK3XG8T0qUoCkJR0qODEJnNuLpEZIZ4Usr0TCotsExFl1mDXR8kfy4OHz6CnrIwkknFZrOlgQFEZnC4Skv/A+WoJliWhWma
uHNclpHQEBP9dTwRX8Xj7N+2ai+CDrtU9gAAAABJRU5ErkJggg==
}

image create photo ::img::delete -format GIF -data {
    R0lGODlhEAAQAIABAIQAAP///yH5BAEAAAEALAAAAAAQABAAAAIjjI+pmwAc3HGy
    PUSvqYpuvWQg40FfSVacBa5nN6JYDI3mzRQAOw==
}

image create photo ::img::print -format GIF -data {
    R0lGODlhEAAQALMKAAAAAP///52VunNkl8C82Yl+qldBgq+pyrOzs1fYAP///wAA
    AAAAAAAAAAAAAAAAACH5BAEAAAoALAAAAAAQABAAAARGUMlJKwU4AztB+ODGeUiJ
    fGLlgeEYmGWQXmx7aXgmAUTv/74N4EAsGhOJg1DAbDqbwoJ0Sp0KB9isNis0eL/g
    ryhH5pgnEQA7
}

image create photo ::img::view -format GIF -data {
    R0lGODlhEAAQAKIHAP///wwMDAAAAMDAwNnZ2SYmJmZmZv///yH5BAEAAAcALAAA
    AAAQABAAAANMKLos90+ASamDRxJCgw9YVnlDOXiQBgRDBRgHKE6sW8QR3doPKK27
    yg33q/GIOhdg6OsEJzeZykiBSUcs06e56Xx6np8ScIkFGuhQAgA7
}

image create photo ::img::avec_det -data {
iVBORw0KGgoAAAANSUhEUgAAAAgAAAAICAIAAABLbSncAAAACXBIWXMAAA7EAAAO
xAGVKw4bAAAAB3RJTUUH4wEVCRAgeN5EhwAAAB1pVFh0Q29tbWVudAAAAAAAQ3Jl
YXRlZCB3aXRoIEdJTVBkLmUHAAAAJklEQVQI12P8//8/AzbAxEAywGrU////mTDl
IFwmNH1wBiPJrgIABS4X85g6J8IAAAAASUVORK5CYII=
}

image create photo ::img::avec_det1 -data {
iVBORw0KGgoAAAANSUhEUgAAAAoAAAAFCAYAAAB8ZH1oAAAABmJLR0QA/wD/AP+g
vaeTAAAACXBIWXMAAA7EAAAOxAGVKw4bAAAAB3RJTUUH4wEUEhEE9Qm64wAAAB1p
VFh0Q29tbWVudAAAAAAAQ3JlYXRlZCB3aXRoIEdJTVBkLmUHAAAAQElEQVQI133K
MQrAMAwEwTXkzyqlUj92uakCSiA+2OqG7hY4VlWicsJVpQr6jx/0gl88kcrFWESs
vbcAmbnmdwPsWlLtRYEjcgAAAABJRU5ErkJggg==
}

image create photo ::img::not_det -data {
iVBORw0KGgoAAAANSUhEUgAAAAgAAAAICAIAAABLbSncAAAACXBIWXMAAA7EAAAO
xAGVKw4bAAAAB3RJTUUH4wEVCQ4Al/FbkAAAAB1pVFh0Q29tbWVudAAAAAAAQ3Jl
YXRlZCB3aXRoIEdJTVBkLmUHAAAAMUlEQVQI12P8//8/AwMDIyMjhAEHTHAWIyMj
dgk0ORQJZDl0CZw64E5gwiqKIoHmXADVjg8Nbya7dgAAAABJRU5ErkJggg==
}

image create photo ::img::not_det1  -data {
iVBORw0KGgoAAAANSUhEUgAAAAUAAAAKCAYAAAB8OZQwAAAABmJLR0QA/wD/AP+g
vaeTAAAACXBIWXMAAA7EAAAOxAGVKw4bAAAAB3RJTUUH4wEUEhIjey5cSwAAAB1p
VFh0Q29tbWVudAAAAAAAQ3JlYXRlZCB3aXRoIEdJTVBkLmUHAAAAQUlEQVQI12WO
MQ7AQAjD3G68OCP8PF1axNFILLaIgiTbZh6AtwDwFg2nuF7QkcTNSkSc71Xlo/MD
DSewDZn5G/8A70VdxL6tU7QAAAAASUVORK5CYII=
}

image create photo ::img::logo  -data {
iVBORw0KGgoAAAANSUhEUgAAADIAAAA+CAYAAABp/UjKAAAABmJLR0QA/wD/AP+gvaeTAAAACXBIWXMAAAsTAAALEwEAmpwYAAAAB3RJTUUH4wEUDxMyHHt92wAAAB1p
VFh0Q29tbWVudAAAAAAAQ3JlYXRlZCB3aXRoIEdJTVBkLmUHAAANbElEQVRo3tWaW4xd11nHf99aa5/LzJy5e2Z8iWPHxk7SNo1N65KkbpySSyUqQSoFCoLQgITUh6II
FZ7gBYQEojxYUB4IStKKgAp1XYW0sROXWEljx3FaHMdxfI9vccaeiT3juZzbXuvjYe9zmxk7Y4t6zB7t0cw+W/usb323////bQmqqkFRkkP4/3Noul5jBDc6Ps1rBz5g
uhgQ02qI3MQGAISgFNos93/6Ftyp85f523//H85crJKJBGm6UZrNvgkNqVSV2wYibr+1H2etpbOzk74Q4yKTGKIgCLV/5CZYdH1rpXGlWlU6uxzGWFwUOQZ6OnDWY51t
uk8RAVWTPE7CApkioEmciCoqJr2sxF4Z7I6w1uAMioSQuEEDqoklAYtRMBqDGLy6GbtzQ0zAakDwxOLSTY4b3gpAUATBERL3BRSrjaWa4BE8wUYEwFBNrt9gc7wIisdo
BYgaCaugqsnGq+IUCKqEoASj9dASgXUrO1l3Ww8VrwSROaP2F30YVSJr2HN4lHfPlOoRg0ASSEnIOxQ0KBoSVyWJDhIp61b38OjGVTdFpbpcjdl/6hyCJIVUEgckdigu
9VGTm5I9twHwN0/dVQ8G8KlHmn8LJKFVMyKERqmNA5wbm+b4+TGKZZ9WD71iUurHXJ/PPXM+WyCTcZwfL+NVG0uoRZIqqCQeaVxoQJW4GnjqhYP89bM/xVgLYm54ftQW
HLxnaKCfvp5epKkNiE/SIYjiWjrPzC7u8rj2XqwRFrItaqhiogyKTxp1/YNavSUxRKn1EEFDGneiFPIZMs5ihLQp3Wg4WOt9Si7jEFVCCOn6kqSvFafEI6Ep2dPwMQrL
ei19XW0EVUQXyiOKNcLwpZhLk2WSlE/yOqSWiKbltyWD0t0wxvHA+j4e/cIqysWAitxYVJw6RQlksxHPbT/Gi2+em0030j9cffnausSApzOXYVGhAwoB8AuDsxKgQmfO
zcphTaMJBBcQFJvkSEgDTgU11PMl9gB2gTBjwBnwWiFGkCANIzSk8CrgRBW8EkKaGDW0G3v2HBzhUmk/lThgdIEyRARjhQNHRxGNQaO6PxJEknRwJ6pISBFwMEBSFarB
8IPXP+DMlncwzixY8VWFEGJuWzrE8kVdGGnks3iFEBL0qykNVE2tq1WtIAz1tFPIZ7FYggkLwxRD0rnbc1kIcYOPAEEDkq7ZaYpfQj3utE5cMoBaCwINn2hTEuovONEV
LKgKkXhEbB39ygwkXm+IISjNiSAIn/mlXu5c1UvZ+wWj7oKSsYa3j4zy3tlyPfSTSivp5oOTtMVYpEFzBYwV7rt7EY89ePtNgX6ffn4/75093YowVJPQk9QjEhQTFIyk
cCWxTtVz0xweTGh0M0mhSz20UtCLhpAwrhrS93BudJojZ0epVPyChJWmuZqNLMNjpWR9TSpKCEnCpzmiBJJkt03JUwmwZedptrxygiCmFXXeUGMUUY9IDmtcPdmTWlUT
TZqSPfmRJuisjFwqMTI2jjVRg8vfYFai1PhIF71dHfUVzkR+roWOqLZ8vKivm/6+nqSYqS5IzaovWQKoR1skEK2ngkvDcFZnEIG8g3wmpS6yQL09KAahWIFSzBXFdtcg
L+lZg8VG2XhXPw98Zimlqm+B8TfaJ1lnePH1U+x65wIqc4NXVyvHOkNsVRHWLO/mvnXLborqe/jkKLsOBFRtA2M0EFUzZ1dUm4NLE4p7kxxGQNU2BBKZI0fm0r69V157
+ySXpyeoVrROMW80ZxfAOmHvofOJ9FALf50ztHRWUYpj2LFnlP/cdhxxgpk3Z9f/g/qcGlIjeR76+jspdOcQDKqSTApCY01OmpBkszFGhWwUUehoxxhJ+XOqpuj8E3U2
Ubry2mVOFUVR9UQZ0/J5stZQbwtujoqd3GoC2bwjijoTnVUCjXnWlbBwbSeVjIvSHKuFpV5RPrtq4GpTWZ25Udo4Xf3JM+4zGO5YnuXWJe3EsTbhf7lqQBgRSnHg4LFx
xqYC1rTKTDLHE2S+0XYVl7v6k2c8zVlh0/ohfvfLd1GqxPN4aq26CJemyvzF5l2MXp7Czih9V4ksNCgiMm/i0yKKqiriUwXFNj7yEmMcGGPI59wMbNMoezMfLQi5qmJF
IQgaEnJUn8QiWIVgko7tvRIILO3L8ugjK9j+36c58mER6wLWWGKVKwsfoVGCXbNpLVjLR1QrEPuY6bJvURrrUHoOvBAZw2SxQiVI6ujWIiKqVFUZ7Lbcf/cSRibKjE1O
81sPreXz625lcW87W3ecoKsrYrCzjZ+8+SEfjlWInLlivOnsPtLEY3zMtp+eZt/B4UQqqkmVaULJHFi4liPlOObcaBXnJN0cqZflaG0PubNjrFjawdd/Zz3FcsyBI8NM
Tnm+u3UvgwM9/ObDq/n07Ytpz0WcurCT469/QEdHO9Y2r791Ja3ab6jNgxJB4u1D4+wYGcPa+TdDVbBOuXX5EPl8ttUbKN1DOdZ09rF+7QB795/h1T2nILJ0ZLKIVhmf
/IjLpRI/2X2ML21czcZ1yzh3YYqLl2ImynGy8JqOqIqG0DAk1MJKG7GvInT1ttPRma+TKplH9mnaeaPI4n2oK/tBYzrywm8v6eFzdw9x9OQoz27Zy72/vJKHN66hkM8T
ZSzVSpXxqQovv3qQf/juLv7gsQ1865ub2LH7KM/9+H0mpkIL7dBWrDV70GlVqXpPpVK5prGCqmKsJXIGYyT9smQX4yDk2i2FXIZtr7zLF+9ZzaMPfYJsJsIXixRPnKB9
7Vo62rP8/lc+Sz6XYcuP3+Gv/mQJnW05dOYoUGdAFKn3G9NUfpXf2LSKB++7hVJ5/iKEtYbLkxWe/eF+Tg+XiKxBFFQiylOBN948SU97Fmsz/Oo9a8hmMqBQ/NlbHNt4
P0u/8x0WPf44YoRff+Qu9h8aZufuIxw9Pc7lYowRCxIQFQST1kmpeURnS24GbltSYMOdQ9eMlC5PV/j+9oN1t3uThO3i/ohN967i6LERVi3vo6+nLZFnRy9w8ok/pNDf
z8QzT1PY+HlyK1eSMcKnPrmYc+cn+cTqPvYeHOX8pUBQUyfmiUF1F+jcWXudgoOIaSm7EgzOT/OXT97LAxuWMzo2zeJFBUxK1s48+cfkP/oIzWWpvvEGFzZvRhGMUZYN
dDE+WeKhL6zlG1/bQMamrUC0SWuoQZRUL9WmYWgIcOL0CG/ty1Cq+nkbZQyMT3suT3lEJFFoRFFtY+++0yztK5DvyNbHzOe//W2q21/GtbWBKra/j+K/PsfI+nUMPP41
fFUoFPJcvFjkwOEL+CAJEVSpcxNpSKYyC25VY8sPXz7J97a+jRrbJAd9/JBZjdDR10PkXL2KYQ1/90+vMz4yzppPruDYB2Oc3vo85c2bsblco/cruGzER3/6Z2Q++yu8
P1xmsK+D7z3/Fs/84BBtfQMYozBjyGYab0QIRgSpnVTwxJSCoxIMlUDTKenJnNerIYEeiCBiMIBIwLh2xqerrF7ejVRKTD3zNExNzsZcYrDOceaJJzgyPMGqZT1UgkGi
XPrGkiZh2YTLnNRpbiA0wxAMuY4ucu0gJDPE+fOlkISVD8l3aWLY0uW9PLjpTga627hn1/exu19FMm1NQoGSEyEyjulQxp45y+/t+hdW/NGzXJwq8cq+MS5NVJFaNoT0
bSaZCVGacsSIoafT0tNp0SDoPKmuiBCHmOHRMuUyiWCrYEQplj3bdp6g68Br9GzbStUrVMqQzSBAVyZLd2QBxZcUDTHFn+3hnX98hpeqKyiWA0akaQLdCGo3k6FqfSRs
eOSeJXzl4dspl+N5E1RrhInpmL95ag9HTk+Tqb2FJ0Kl4nn+xYPcsWqU/MAdrPvyGuLREfzLL+GiLF2RwyvEk5PYr34VyWY4vO99Xvi3N9i5SLGFDsSZNBFaMaurwQpN
Jfp6SluluxCxuL/juvpIzmry+lRa52vGmPYO/v6k5c4Nj/Opbz5E/N676Av/hclkk5Ijgk5MMP2NJ5nOF3jqWzs4cGyCTFs+0dZCE+qtvx9AoyHOIioEMtH1KSftbRmM
EUQDBkszI8m1ZQn5LCeGp/j6n2/hsbsHuE9rY71kV7M9XTzz0nG27z3BxWIeW+iuaxGtLLYRC+5KokfZw3/86DC7dr+fjhvm2UeAkvecGK4SOVvT01v4CAaKPuL44XF+
bX0uTdp0Q0PADvbxz88doGgs3YN58jlp4Uotm64tWGu2KT7EHDo5yd6fTyLmWjyjiIH+wUEy2QiV0LJTtZm+zTr6BvtpGxhKwy4Zegqg2Rz9t/QzHQuZTDTLiGZKlbzu
Iw3JtAbha7tngiOXaUN67TXPRiQd3SWyjcygxT6tYoLLWogMuAhCCjtUsfk8WIN1EV6vPFuchX7ryn2zd0TI5XPk2/PXLriJ4kOol0LTKlo1dKygOOdIqF9DoyKfxyKN
bn014UET6u0UJa4q1arWAFeTknadkqGQhBRNb3PPccSVwFQMGkWNaFCFfI5qJVD9mEiolj2VkkeD4rIZYcVQnkKbJYpkhnR6vSr2/DZAVfn54Q/5YqETpqeS8qoK2Rwr
B3OUr/L+i3jPxLoVfG6onVxW+F+7L/Vn+4jLbAAAAABJRU5ErkJggg==

}

proc edithex {hex} {
    set c ""
    set l [string length $hex]
    for {set j 0 } { $j < $l} {incr j +2} {
	set c "$c[string range $hex $j $j+1] "
    }
    return [string toupper [string trimright $c]]
}

proc aboutUtil {w} {
    set title [mc "Parse and view the X509 certificate"]
    set content "
	Copyright (c) 2019 Vladimir Orlov 
	email: <vorlov@lissi.ru>
	http://soft.lissi.ru/
	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
	See the GNU General Public License for more details.
	
	[mc {The distribution is here}] (Linux, OS/X, Windows):
	
	http://soft.lissi.ru/ls_product/utils/chainfromcert/
	"
    set content_ru {

    }
    
    catch {destroy $w}
    toplevel $w -padx 4 -pady 4 -bg #c0bab4 
    wm geometry $w +150+100

    wm title $w $title
    wm iconphoto $w ::img::logo
    
    text $w.text -width 66 -height 20 -background #eff0f1 -yscrollcommand [list $w.vsb set] -font {Times 10 bold italic}
    ttk::scrollbar $w.vsb -orient vertical -command [list $w.text yview]

    $w.text tag configure tagAbout -foreground blue -font {Times 10 bold italic}
    $w.text image create end -image creator_small

    frame $w.separ -height 5 -bd 2 -relief groove -bg #ffffd1
    frame $w.butt  -bd 3 -relief groove -bg #c0bab4
    pack $w.butt -expand 1 -fill both -side bottom
    pack $w.separ -expand 1 -fill x -side bottom -pady 4
    pack $w.vsb -side right -fill y -in $w
    pack $w.text -padx 0 -side left
    ttk::button $w.butt.ok -text "Ok" -command "destroy $w"
    pack $w.butt.ok -side right -padx 20 -pady 4 -in $w.butt
    
    $w.text insert end "\t\t$title" tagAbout
    $w.text insert end $content
    $w.text configure -state disabled
    
    wm protocol $w WM_DELETE_WINDOW ";"
    
    grab set $w
    tkwait window $w
}


proc addSeeDismiss {w show} {
    global typesys
    ttk::frame $w -pad 3
    ttk::style configure TFrame -background #e8e8e7
    ttk::style configure TButton -background #e2e2e1
    ttk::style configure TButton  -font TkFixedFont
    ttk::button $w.cancel -text [mc "Cancel"] \
	-image ::img::delete -compound left \
	-command exit
    ttk::menubutton $w.loadca -text [mc "Additionaly"] -menu $w.loadca.menu -direction above
    menu $w.loadca.menu -tearoff 0
    $w.loadca.menu add command -label [mc "Load chain CA"] -command {::loadchain}
    $w.loadca.menu add command -label [mc "Load CRL"]  -command {::loadcrl}
    set aa "$w.loadca.menu add command -label {[mc "Save certificate"]} -command {exportparsecert \"\" $show }"
    set com [subst "$aa"]
    eval $com
    
    set bb [mc "Load certificate"]
    set aa "ttk::button $w.loadcert -text \"$bb\" -image ::img::view -compound left -command {::fileWithCert $show \"\"}"
    set com [subst "$aa"]
    eval $com
    $w.loadca.menu add separator
    $w.loadca.menu add command -label [mc "About"] -command {aboutUtil .about}
    set buttons [list x $w.loadca $w.loadcert $w.cancel]
    grid {*}$buttons -padx 4 -pady 4
    grid columnconfigure $w 0 -weight 1
    switch $typesys {
	win32        { 
	    ;
	}
	x11 { 
	    $w.loadca configure -style My.TButton
	    $w.cancel configure -style My.TButton
	    $w.loadcert configure -style My.TButton
	}
	aqua { 
	    foreach b [lrange $buttons 1 end] {$b configure -takefocus 0}
	    grid configure {*}$buttons -pady {10 12}
	    grid configure [lindex $buttons 1] -padx {16 4}
	    grid configure [lindex $buttons end] -padx {4 18}
	}
}

    return $w
}

#ttk::style theme use clam
#ttk::style theme use clearlooks
ttk::style configure TLabel -background #ffffff
set myfont [font configure TkDefaultFont]
set myfont [lreplace $myfont 5 5 bold]
set com "font create TkDefaultFontBold $myfont"
set com [subst $com]
eval $com
set myfont [lreplace $myfont 3 3 12]
set com "font create TkFontForFirst $myfont"
set com [subst $com]
eval $com

set w .text
wm geometry . 65x16+150+100
wm title . [mc "Parse and view the X509 certificate"]
wm minsize . 47 16
wm iconphoto . ::img::logo
frame $w -bd 1 -bg #ffffff
pack $w -expand 1 -fill both 


set btns [addSeeDismiss $w.buttons $w]
pack $btns -side bottom -fill x
global avecdet
set avecdet 1
proc changelang {w} {
    global loadfile
    global lang
    if {$lang == "ru"} {
	set lang "en"
	$w.tit.right.lang configure -image  ::img::us_32x32 -compound left
    } else {
	set lang "ru"
	$w.tit.right.lang configure -image  ::img::ru_32x32 -compound left
    }
    focus -force $w.text
    ::msgcat::mclocale $lang
    set title [mc "Parse and view the X509 certificate"]
    wm title . $title
    $w.buttons.loadca configure -text [mc "Additionaly"]
    $w.buttons.cancel configure -text [mc "Cancel"]
    $w.buttons.loadcert configure -text [mc "Load certificate"]
    $w.tit.left.caption configure -text [mc "Details"]
    if {$loadfile != ""} {
	::fileWithCert $w $loadfile
    } else {
	set ident [mc "Identity"]
	set verca [mc "Verified by"]
	set oidval "[mc "Subject Name"]"
	set dateex "[mc "Expires"]"

	$w.tit.left.l1 configure -text "$oidval"
	$w.tit.left.l2 configure -text "$ident: $oidval"
	set oidval "[mc "Issuer Name"]"
	$w.tit.left.l3 configure -text "$verca: $oidval"
	set oidval "[mc "Not Valid After"]"
	$w.tit.left.l4 configure -text "$dateex: $oidval"
    }
}

proc typeview {w} {
    global avecdet
    global loadfile
    if {$avecdet == 0} {
	set avecdet 1
	$w.tit.left.caption configure -image ::img::avec_det    
	$w.tit.left.caption configure -state normal    
	pack forget $w.text
	if {$loadfile != "" } {
	    pack $w.scroll -side right -fill y -padx {0 5}
	}
	pack $w.text -expand yes -fill both -side bottom
	$w.text configure -foreground #000000
	focus -force $w.text
    } else {
	set avecdet 0
	$w.tit.left.caption configure -image ::img::not_det   
	$w.tit.left.caption configure -state normal    
	$w.text configure -foreground #ffffff
	pack forget $w.scroll
	focus -force $w.text
    }

}

proc chainfromcert {cert dir} {
    global count
    if {$cert == "" } {
	exit
    }
    set asndata [cert_to_der $cert]
    if {$asndata == "" } {
#Файл содержит все что угодно, только не сертификат
	return -1
    }
    array set cert_parse [::pki::x509::parse_cert $asndata]
    array set extcert $cert_parse(extensions)
    if {![info exists extcert(1.3.6.1.5.5.7.1.1)]} {
#В сертификате нет расширений
	return 0
    }
    set a [lindex $extcert(1.3.6.1.5.5.7.1.1) 0]
#Читаем ASN1-последовательность расширения в Hex-кодировке
    set b [lindex $extcert(1.3.6.1.5.5.7.1.1) 1]
#Переводим в двоичную кодировку
    set c [binary format H* $b]
#Sequence 1.3.6.1.5.5.7.1.1
    ::asn::asnGetSequence c c_par_first
#Цикл перебора значений в засширении 1.3.6.1.5.5.7.1.1
    while {[string length $c_par_first] > 0 } {
#Выбираем очередную последовательность (sequence)
	::asn::asnGetSequence c_par_first c_par
#Выбираем oid из последовательности
	::asn::asnGetObjectIdentifier c_par c_type
	set tas1 [::pki::_oid_number_to_name $c_type]
#Выбираем установленное значение
	::asn::asnGetContext c_par c_par_two
#Ищем oid с адресом корневого сертификата
	if {$tas1 == "1.3.6.1.5.5.7.48.2" } {
#Читаем очередной корневой сертификат
	    set certca [readca $c_par $dir]
	    if {$certca == ""} {
#Прочитать сертификат не удалось. Ищем следующую точку с сертификатом
		continue
	    } else {
#Сохраняем корневой сертификат в указанном каталоге
		set f [file join $dir [file tail $c_par]]
		set fd [open $f w]
		chan configure $fd -translation binary
		puts -nonewline $fd $certca 
		close $fd
		lappend ::listcert $f
		incr count
#ПОДЫМАЕМСЯ по ЦЕПОЧКЕ СЕРТИФИКАТОВ ВВЕРХ
		chainfromcert $certca $dir
		continue
	    }
	} elseif {$tas1 == "1.3.6.1.5.5.7.48.1" } {
#	    puts "OCSP server (oid=$tas1)=$c_par"
	}
    }
# Цепочка закончилась
    return $count
}


proc readca {url dir} {
    set cer ""
#Читаем сертификат в бинарном виде
    if {[catch {set token [http::geturl $url -binary 1]
#получаем статус выполнения функции
	set ere [http::status $token]
	if {$ere == "ok"} {
#Получаем код возврата с которым был прочитан сертификат
	    set code [http::ncode $token]
	    if {$code == 200} {
#Сертификат успешно прочитан и будет созвращен
                set cer [http::data $token]
    	    } elseif {$code == 301 || $code == 302} {
#Сертификат перемещен в другое место, получаем его 
         		set newURL [dict get [http::meta $token] Location]
#Читаем сертификат с другого сервера
          		set cer [readca $newURL $dir]
    	    } else {
#Сертификат не удалось прочитать
        	set cer ""
    	    }
        } 
    } error]} {
#Сертификат не удалось прочитать, нет узла в сети
	set cer ""
    }
    return $cer
}

proc ::loadchain {} {
    global count
    global loadfile
    global home
    if {$::chaincert == "" } {
	if {$loadfile == ""} {
	    tk_messageBox -title [mc "Load chain CA"] -icon info -message [mc "No certificate selected"]
	    return
	}
	tk_messageBox -title [mc "Load chain CA"] -icon info -message [mc "No information"]
	return
    }
    set fd [open $loadfile]
    chan configure $fd -translation binary
    set data [read $fd]
    close $fd
    set dir [tk_chooseDirectory -initialdir $home -title [mc "Choos directory for chain"]]
    if {$dir == ""} {
	return
    }
    set ::listcert {}
    set count 0
    set depth [chainfromcert $data $dir]
    if {$depth == -1} {
	puts "Bad file with certificate=$file"
    }
    set lenchain [llength $::listcert]
    set a ""
    set a "[mc {Files with CA certificates}]:\n\n"
    foreach b $::listcert {
	set a "$a  $b\n"
    }
    tk_messageBox -title [mc "Load chain CA"] -icon info -message "[mc {Length chain CA}] - $lenchain" -detail $a
    return
}

proc ::loadcrl {} {
    global count
    global loadfile
    global home
    if {$::crlfile == "" } {
	if {$loadfile == ""} {
	    tk_messageBox -title [mc "Load CRL"] -icon info -message [mc "No certificate selected"]
	    return
	}
	tk_messageBox -title [mc "Load CRL"] -icon info -message [mc "No information"]
	return
    }
    set dir [tk_chooseDirectory -initialdir $home -title [mc "Load CRL"]]
    if {$dir == ""} {
	return
    }
    set filecrl ""
    set pointcrl ""
    foreach pointcrl $::crlfile {
	    set filecrl [readca $pointcrl $dir]
	    if {$filecrl != ""} {
		set f [file join $dir [file tail $pointcrl]]
		set fd [open $f w]
		chan configure $fd -translation binary
		puts -nonewline $fd $filecrl
		close $fd
		set filecrl $f
		break
	    } 
#Прочитать CRL не удалось. Берем следующую точку с CRL
    }
    if {$filecrl != ""} {
	tk_messageBox -title [mc "Load CRL"] -icon info -message "[mc {Load CRL from}]\n$pointcrl" -detail "  [mc {File with CRL}]:\n$filecrl"
    } else {
	tk_messageBox -title [mc "Load CRL"] -icon error -message [mc "Cannot load CRL"]
    }
}

proc ::pki::x509::parse_cert_pubkeyinfo {cert_hex} {
	array set ret [list]
	set wholething [binary format H* $cert_hex]
	::asn::asnGetSequence wholething cert

	::asn::asnPeekByte cert peek_tag
	if {$peek_tag != 0x02} {
		# Version number is optional, if missing assumed to be value of 0
		::asn::asnGetContext cert - asn_version
		::asn::asnGetInteger asn_version ret(version)
	}

	::asn::asnGetBigInteger cert ret(serial_number)
	::asn::asnGetSequence cert data_signature_algo_seq
		::asn::asnGetObjectIdentifier data_signature_algo_seq ret(data_signature_algo)
	::asn::asnGetSequence cert issuer
	::asn::asnGetSequence cert validity
		::asn::asnGetUTCTime validity ret(notBefore)
		::asn::asnGetUTCTime validity ret(notAfter)
	::asn::asnGetSequence cert subject
	::asn::asnGetSequence cert pubkeyinfo
#Me
	binary scan $pubkeyinfo H* ret(pubkeyinfo)
	return $ret(pubkeyinfo)
}


proc parse_key_gost {pubkeyinfo_hex} {
    array set ret [list]

    set pubkeyinfo [binary format H* $pubkeyinfo_hex]

    ::asn::asnGetSequence pubkeyinfo pubkey_algoid
	::asn::asnGetObjectIdentifier pubkey_algoid ret(pubkey_algo)
    ::asn::asnGetBitString pubkeyinfo pubkey
    set pubkey [binary format B* $pubkey]
    binary scan $pubkey H* ret(pubkey)
    ::asn::asnGetSequence pubkey_algoid pubalgost
#OID - параметра
    ::asn::asnGetObjectIdentifier pubalgost ret(paramkey)
#OID - Функция хэша
    ::asn::asnGetObjectIdentifier pubalgost ret(hashkey)
    return [array get ret]

}

proc issuerpol {iss_hex} {
    array set ret [list]
    set iss [binary format H* $iss_hex]
    ::asn::asnGetSequence iss iss_pol
    for {set i 1} {[string length $iss_pol] > 0}  {incr i} {
	::asn::asnGetUTF8String iss_pol ret(isspol$i)
    }
    return [array get ret]
}
proc subjectpol {iss_hex} {
    array set ret [list]
    set iss [binary format H* $iss_hex]
    for {set i 1} {[string length $iss] > 0}  {incr i} {
	::asn::asnGetUTF8String iss ret(isspol$i)
    }
    return [array get ret]
}

proc extku {ku_hex} {
    set typeku {"Digital signature" "Non-Repudiation" "Key encipherment" "Data encipherment" "Key agreement" "Certificate signature" "CRL signature" "Encipher Only" "Decipher Only"}
    set ku [binary format H* $ku_hex]
    ::asn::asnGetBitString ku ku_bin
    set ret {}
    for {set i 0} {$i < [string length $ku_bin]}  {incr i} {
	if {[string range $ku_bin $i $i] > 0 } {
	    lappend ret [lindex $typeku $i]
	}
    }
    return $ret
}

proc extpol {pol_hex} {
    set pol [binary format H* $pol_hex]
    set ret {}
    ::asn::asnGetSequence pol oid_pol
    for {set i 1} {[string length $oid_pol] > 0}  {incr i} {
	::asn::asnGetSequence oid_pol oid_pol1
	::asn::asnGetObjectIdentifier oid_pol1 ret1
	lappend ret $ret1
    }
    return $ret
}

proc extkeyuse {keyuse_hex} {
    set use [binary format H* $keyuse_hex]
    set ret {}
    ::asn::asnGetSequence use oid_use
    for {set i 1} {[string length $oid_use] > 0}  {incr i} {
	::asn::asnGetObjectIdentifier oid_use ret1
	lappend ret $ret1
    }
    return $ret
}

proc chainocsp {chain_hex} {
    set chain [binary format H* $chain_hex]
    set ret {}
    ::asn::asnGetSequence chain c_par_first
    while {[string length $c_par_first] > 0 } {
#Выбираем очередную последовательность (sequence)
	::asn::asnGetSequence c_par_first c_par
#Выбираем oid из последовательности
	::asn::asnGetObjectIdentifier c_par c_type
	set tas1 [::pki::_oid_number_to_name $c_type]
#Выбираем установленное значение
	::asn::asnGetContext c_par c_par_two
#Ищем oid с адресом корневого сертификата
	if {$tas1 == "1.3.6.1.5.5.7.48.2" } {
#Читаем очередной корневой сертификат
	    lappend ret "CA Issuers"
	    lappend ret $c_par
	} elseif {$tas1 == "1.3.6.1.5.5.7.48.1" } {
	    lappend ret "OCSP"
	    lappend ret $c_par
	}
    }
# Цепочка закончилась
    return $ret
}

proc crlpoints {crl_hex} {
    set crl [binary format H* $crl_hex]
    set ret {}
    ::asn::asnGetSequence crl p_crl_first
    set lencontspec 4
    while {[string length $p_crl_first] > 0 } {
#Выбираем очередную последовательность (sequence)
	::asn::asnGetSequence p_crl_first c_par
#Пропускаем contextspecific - 0xA0
	::asn::asnGetBytes c_par $lencontspec context_0xa0
	::asn::asnGetContext c_par ux
#	puts $c_par
	lappend ret $c_par
    }
    return $ret
}

proc altname {alt_hex} {
    set ret {}
    set listname [binary format H* $alt_hex]
    ::asn::asnGetSequence listname names
    while {[string length $names] > 0 } {
	::asn::asnGetByte names tag
	if {$tag == 0x82 || $tag == 0x81} {
#0x82 DNS Name ; 0x81 - RFC 822
########################
	    ::asn::asnGetByte names taglen
	    set len82 [format "%i" $taglen]
	    incr len82 -1
	    if {$tag == 0x82 } {
#Читаем очередной корневой сертификат
		lappend ret "DNS Name"
	    } elseif {$tag == 0x81 } {
		lappend ret "RFC822 Name"
	    } else {
		lappend ret [forma "0x%2x" $tag]
	    }
	    lappend ret [string range $names 0 $len82]
	    incr len82
	    set a [string range $names $len82 end]
	    set names $a
	} 
    }
    return $ret
}

proc keyperiod {per_hex} {
    set per [binary format H* $per_hex]
    array set ret [list]
    ::asn::asnGetSequence per validaty
    ::asn::asnGetByte validaty tag
    if {$tag == 0x80} {
	::asn::asnGetByte validaty tag
	set len80 [format "%i" $tag]
	incr len80 -1
	set ret(notBefore) [string range $validaty 0 $len80]
	incr len80
	set a [string range $validaty $len80 end]
	set validaty $a
	::asn::asnGetByte validaty tag
    } else {
	set ret(notBefore) ""
    }
    if {$tag != 0x81} {
	set ret(After) ""
	return [array get ret]
    }
    ::asn::asnGetByte validaty tag
    set len81 [format "%i" $tag]
    incr len81 -1
    set ret(notAfter) [string range $validaty 0 $len81]
    incr len81
    return [array get ret]
}

proc autkeyid {autkey_hex} {
    set autkey [binary format H* $autkey_hex]
    array set ret [list]
    set autkey [binary format H* $autkey_hex]
    array set ret [list]
    ::asn::asnGetSequence autkey fullkey
#Чтение KEY ID
    ::asn::asnGetContext fullkey - cont
    binary scan $cont H* ret(authKeyID)
    while {[string length $fullkey]} {
	::asn::asnGetByte fullkey tag1
	::asn::asnGetLength fullkey tag
	set len80 [format "0x%x" $tag]
	set ee [string range $fullkey 0 0]
	if { $ee == 0} {
	    ::asn::asnGetSequence fullkey autdn
	    set ret(issuer) [::pki::x509::_dn_to_string $autdn]
	    ::asn::asnGetContext fullkey - sernum
	    binary scan $sernum H* ret(sernum)
	    break
	}
    }

    return [array get ret]
}

proc cert_to_der {data} {
    if {[string first "-----BEGIN CERTIFICATE-----" $data] != -1} {
	set data [string map {"\r\n" "\n"} $data]
    }
    array set parsed_cert [::pki::_parse_pem $data "-----BEGIN CERTIFICATE-----" "-----END CERTIFICATE-----"]
    set asnblock $parsed_cert(data)
    return $asnblock
}

proc del_comma {ldn} {
    set lret {}
    set ff ""
    foreach el $ldn {
	if {[string first "=" $el] != -1} {
	    if {$ff != ""} {
		lappend lret $ff
	    }
	    set ff $el
	} else {
	    set ff "$ff,$el"
	}
    }
    lappend lret $ff
    return $lret
}

proc ::fileWithCert {w file} {
    global avecdet
    global home
    global loadfile
    global dn_fields
#Файл с сертификатм из командной строки
    set comline 1
    
    set typeFile {
	{"Сертификат в формате DER"    .der}
	{"Сертификат в формате DER"    .cer}
	{"Сертификат в формате PEM"    .pem}
	{"Сертификат в формате PEM"    .crt}
	{"Любой файл"    *}
    }
    if {$file == "" } {
	set lastdir $home
	if {$loadfile != "" } {
	    set lastdir [file dirname $loadfile]
	}
	set titleS "Выберите файл с сертификатом"
	set file [tk_getOpenFile -title $titleS -filetypes $typeFile -initialdir "$lastdir"]
	set comline 0
    }
    set ident [mc "Identity"]
    set verca [mc "Verified by"]

    if {![string compare $file ""]} {
	puts "Файл не выбран"
	return ""
    }
    set fd [open $file]
    chan configure $fd -translation binary
    set data [read $fd]
    close $fd
    set asndata [cert_to_der $data]
    if {$asndata == "" } {
	tk_messageBox -title [mc "Load certificate"] -icon error -message "$file" -detail "Выбранный файл не содержит сертификата"
	return
    }
    if {[catch {array set cert_parse [::pki::x509::parse_cert $asndata]} rc]} {
	tk_messageBox -title [mc "Load certificate"] -icon error -message "$file" -detail "Выбранный файл не содержит сертификата"
	return
    }
#Читаем публичный ключ
    set cert_parse(pubkeyinfo) [::pki::x509::parse_cert_pubkeyinfo $cert_parse(cert)]

    set loadfile $file
#parray cert_parse
    set lsub [split $cert_parse(subject) ","]
    set lsub [del_comma $lsub]

    $w.text delete 0.0 end
    $w.text insert end [mc "Subject Name"] bold
    $w.text insert end "\n"
    foreach a $lsub {
	set ind [string first "=" $a]
	if {$ind == -1 } { continue }
	
	set oidsub [string trim [string range $a 0 $ind-1]]
	if {[info exists dn_fields($oidsub)]} {
	    set nameoid " ([mc $dn_fields($oidsub)])"
	} else {
	    set nameoid ""
	}

	set oidval "[mc [string trim [string range $a $ind+1 end]]]"
	
	if {$oidsub == "CN"} {
	    $w.tit.left.l1 configure -text "$oidval"
	    $w.tit.left.l2 configure -text "$ident: $oidval"
	}
	set oidsub "$oidsub$nameoid"
	$w.text insert end "\t$oidsub\t$oidval\n"  margins1
    }
    $w.text insert end [mc "Issuer Name"] bold
    $w.text insert end "\n"
    set liss [split $cert_parse(issuer) ","]
    set liss [del_comma $liss]

    foreach a $liss {
	set ind [string first "=" $a]
	if {$ind == -1 } { continue }
	set oidsub [string trim [string range $a 0 $ind-1]]
	if {[info exists dn_fields($oidsub)]} {
	    set nameoid " ([mc $dn_fields($oidsub)])"
	} else {
	    set nameoid ""
	}

	set oidval "[mc [string trim [string range $a $ind+1 end]]]"

	if {$oidsub == "CN"} {
	    $w.tit.left.l3 configure -text "$verca: $oidval"
	}
	set oidsub "$oidsub$nameoid"
	$w.text insert end "\t$oidsub\t$oidval\n"  margins1
    }
    $w.text insert end [mc "Issued Certificate"] bold
    $w.text insert end "\n"
    $w.text insert end "\t[mc "Version"]:\t$cert_parse(version)\n"  margins1
    $w.text insert end "\t[mc "Serial Number"]:\t$cert_parse(serial_number)\n"  margins1
    set t $cert_parse(notAfter)
    set notafter [clock format $t -format "%d/%m/%Y %R %Z"]
    set t $cert_parse(notBefore)

    set notbefore [clock format $t -format "%d/%m/%Y %R %Z"]
    $w.text insert end "\t[mc "Not Valid Before"]:\t$notbefore\n"  margins1
    $w.text insert end "\t[mc "Not Valid After"]:\t$notafter\n"  margins1
    set ver [mc "Expires"]
    $w.tit.left.l4 configure -text "$ver: $notafter"
    $w.text insert end [mc "Certificate Fingerprints"] bold
    $w.text insert end "\n"
    set fingerprint_sha1 [::sha1::sha1 $asndata]
    set fingerprint_sha256 [::sha2::sha256 $asndata]

    $w.text insert end "\t[mc "SHA1"]:\t[edithex $fingerprint_sha1]\n"  margins11
    $w.text insert end "\t[mc "SHA256"]:\t[edithex $fingerprint_sha256]\n"  margins11
    $w.text insert end [mc "Public Key Info"] bold
    $w.text insert end "\n"
    if {[string range $cert_parse(pubkey_algo) 0 7] == "1.2.643." || [string range $cert_parse(pubkey_algo) 0 7] == "ГОСТ Р 3" || [string range $cert_parse(pubkey_algo) 0 7] == "GOST R 3"} {
	$w.text insert end "\t[mc "Key Algorithm"]:\t[mc $cert_parse(pubkey_algo)]\n"  margins1
	$w.text insert end "\t[mc "Key Parameters"]:\n"  margins1
	array set ret [parse_key_gost $cert_parse(pubkeyinfo)]
	$w.text insert end "\t[mc "sign param"]:\t[mc $ret(paramkey)]\n"  margins2
	$w.text insert end "\t[mc "hash param"]:\t[mc $ret(hashkey)]\n"  margins2
	set sek 4
	if {[string range $ret(pubkey) 2 3] != 40} {
	    set sek 6
	}
	set pk [edithex [string range $ret(pubkey) $sek end]]
	$w.text insert end "\t[mc "Public Key"]:\t$pk\n"  margins11
	
    } else {
	if {[string range $cert_parse(pubkey_algo) 0 2] == "rsa" } {
	    $w.text insert end "\t[mc "Key Algorithm"]:\tRSA\n"  margins1
	    $w.text insert end "\t[mc "Key Size"]:\t$cert_parse(l)\n"  margins2
	    $w.text insert end "\t[mc "Public Key"]:\t[edithex $cert_parse(pubkey)]\n"  margins11
	} else {
	    $w.text insert end "\t[mc "Key Algorithm"]:\t$cert_parse(pubkey_algo)\n"  margins1
	    $w.text insert end "\t[mc "Key Info"]:\t[edithex $cert_parse(pubkeyinfo)]\n" margins11
	}
    }
    array set extcert $cert_parse(extensions)
    if {[info exists extcert(id-ce-basicConstraints)]} {
	$w.text insert end [mc "Basic Constraints"] bold
	$w.text insert end "\n"
	set basic $extcert(id-ce-basicConstraints)
	if {[lindex $basic 1] == 1} {
	    set typecert [mc "Yes"]
	} else {
	    set typecert [mc "No"]
	}
	$w.text insert end "\t[mc "Certificate Authority"]:\t$typecert\n"  margins1
	if {[lindex $basic 2] == -1} {
	    set lencert [mc "Unlimited"]
	} else {
	    set lencert [lindex $basic 2]
	}
	$w.text insert end "\t[mc "Max Path Length"]:\t$lencert\n"  margins1
	if {[lindex $basic 0] == 1} {
	    set critcert [mc "Yes"]
	} else {
	    set critcert [mc "No"]
	}
	$w.text insert end "\t[mc "Critical"]:\t$critcert\n"  margins1
	unset extcert(id-ce-basicConstraints)
    }
#  1 false -1
# первое поле критичность 1 - Да, false - нет    
# второе поле УЦ 1 - Да, false - нет
#Третье поле - длина пути : -1 - неограниченный, или значение 0 и т.д.
    if {[info exists extcert(1.2.643.100.112)]} {
#issuerSignTools
	array set pol [issuerpol [lindex $extcert(1.2.643.100.112) 1]]
	$w.text insert end [mc "issuerSignTool"] bold
	$w.text insert end "\n"
	$w.text insert end "\t[mc "Name CKZI"]:\t$pol(isspol1)\n"  margins1
	$w.text insert end "\t[mc "Name CA"]:\t$pol(isspol2)\n"  margins1
	$w.text insert end "\t[mc "Certificate SKZI CA"]:\t$pol(isspol3)\n"  margins1
	$w.text insert end "\t[mc "Certificate CA"]:\t$pol(isspol4)\n"  margins1
	unset extcert(1.2.643.100.112)
    }
    if {[info exists extcert(1.2.643.100.111)]} {
#subjectSignTools
	array set pol [subjectpol [lindex $extcert(1.2.643.100.111) 1]]
	$w.text insert end [mc "subjectSignTool"] bold
	$w.text insert end "\n"
	$w.text insert end "\t[mc "User CKZI"]:\t$pol(isspol1)\n"  margins1
	unset extcert(1.2.643.100.111)
    }
    if {[info exists extcert(id-ce-keyUsage)]} {
	$w.text insert end [mc "Key Usage"] bold
	$w.text insert end "\n"

	set ku [extku [lindex $extcert(id-ce-keyUsage) 1]]
	$w.text insert end "\t[mc "Usages"]:\t[mc [lindex $ku 0]]\n"  margins1
	for {set i 1 } { $i < [llength $ku] } {incr i} {
	    $w.text insert end "\t\t[mc [lindex $ku $i]]\n"  margins1
	}
	if {[lindex $extcert(id-ce-keyUsage) 0] == 1} {
	    set critcert [mc "Yes"]
	} else {
	    set critcert [mc "No"]
	}
	$w.text insert end "\t[mc "Critical"]:\t$critcert\n"  margins1
	unset extcert(id-ce-keyUsage)
    }
    if {[info exists extcert(id-ce-certificatePolicies)]} {
	$w.text insert end [mc "Certificate Policies"] bold
	$w.text insert end "\n"
	set lpol [extpol [lindex $extcert(id-ce-certificatePolicies) 1]]
	$w.text insert end "\t[mc "Policy Name"]:\t[mc [::pki::_oid_number_to_name [lindex $lpol 0]]]\n"  margins1
	for {set i 1 } { $i < [llength $lpol] } {incr i} {
	    $w.text insert end "\t\t[mc [::pki::_oid_number_to_name [lindex $lpol $i]]]\n"  margins1
	}
	if {[lindex $extcert(id-ce-certificatePolicies) 0] == 1} {
	    set critcert [mc "Yes"]
	} else {
	    set critcert [mc "No"]
	}
	$w.text insert end "\t[mc "Critical"]:\t$critcert\n"  margins1
	unset extcert(id-ce-certificatePolicies)
    }
    if {[info exists extcert(id-ce-subjectKeyIdentifier)]} {
	$w.text insert end [mc "Subject Key Identifier"] bold
	$w.text insert end "\n"
	$w.text insert end "\t[mc "Key ID"]:\t[edithex [string range [lindex $extcert(id-ce-subjectKeyIdentifier) 1] 4 end]]\n"  margins1
	unset extcert(id-ce-subjectKeyIdentifier)
    }
    if {[info exists extcert(id-ce-privateKeyUsagePeriod) ]} {
	$w.text insert end [mc "Key Usage Period"] bold
	$w.text insert end "\n"
	array set keyperiod [keyperiod [lindex $extcert(id-ce-privateKeyUsagePeriod) 1]]
	set t $keyperiod(notBefore)
	set year [string range $t 0 3]
	set month [string range $t 4 5]
	set day [string range $t 6 7]
	set hour [string range $t 8 9]
	set minute [string range $t 10 11]

	set notbefore "$day/$month/$year $hour:$minute"

	set t $keyperiod(notAfter)
	set notafter "[string range $t 6 7]/[string range $t 4 5]/[string range $t 0 3] [string range $t 8 9]:[string range $t 10 11]"
	$w.text insert end "\t[mc "Not Valid Before"]:\t$notbefore\n"  margins1
	$w.text insert end "\t[mc "Not Valid After"]:\t$notafter\n"  margins1

	if {[lindex $extcert(id-ce-privateKeyUsagePeriod) 0] == 1} {
	    set critcert [mc "Yes"]
	} else {
	    set critcert [mc "No"]
	}
	$w.text insert end "\t[mc "Critical"]:\t$critcert\n"  margins1
	unset extcert(id-ce-privateKeyUsagePeriod)
    }
    if {[info exists extcert(id-ce-authorityKeyIdentifier) ]} {
	$w.text insert end [mc "Certificate Authority Key Identifier"] bold
	$w.text insert end "\n"
	array set autkey [autkeyid [lindex $extcert(id-ce-authorityKeyIdentifier) 1]]
	$w.text insert end "\t[mc "Key ID"]:\t[edithex $autkey(authKeyID)]\n"  margins1
	if {[info exists autkey(issuer) ] } {
	    $w.text insert end "\t[mc "Directory Name"]:\t$autkey(issuer)\n"  margins1
	}
	if {[info exists autkey(sernum) ]} {
	    $w.text insert end "\t[mc "Serial Number"]:\t[edithex $autkey(sernum)]\n"  margins1
	}
	unset extcert(id-ce-authorityKeyIdentifier)
    }
    if {[info exists extcert(2.5.29.37) ]} {
	$w.text insert end [mc "Extended Key Usage"] bold
	$w.text insert end "\n"
	set listusage [extkeyuse [lindex $extcert(2.5.29.37) 1]]
	set oidt [string map {" " "."} [lindex $listusage 0]]
	if {[info exists ::payoid($oidt) ]} {
	    set poid " ($::payoid($oidt))"
	} else {
	    set poid ""
	}
	$w.text insert end "\t[mc "Allowed Purposes"]:\t$oidt$poid\n"  margins1
	for {set i 1 } { $i < [llength $listusage] } {incr i} {
	    set oidt [string map {" " "."} [lindex $listusage $i]]
	    if {[info exists ::payoid($oidt) ]} {
		set poid " ($::payoid($oidt))"
	    } else {
		set poid ""
	    }
	    $w.text insert end "\t\t$oidt$poid\n"  margins1
	}
	if {[lindex $extcert(2.5.29.37) 0] == 1} {
	    set critcert [mc "Yes"]
	} else {
	    set critcert [mc "No"]
	}
	$w.text insert end "\t[mc "Critical"]:\t$critcert\n"  margins1
	unset extcert(2.5.29.37)
    } 
    set ::chaincert ""
    if {[info exists extcert(1.3.6.1.5.5.7.1.1)]} {
	$w.text insert end [mc "Authority information Accesss"] bold
	$w.text insert end "\n"
	set listchain [chainocsp [lindex $extcert(1.3.6.1.5.5.7.1.1) 1]]
	foreach {a b} $listchain {
	    $w.text insert end "\t[mc $a]:\tURI:$b\n"  margins1
	}
	set ::chaincert [lindex $extcert(1.3.6.1.5.5.7.1.1) 1]
	unset extcert(1.3.6.1.5.5.7.1.1)
    } 

    set ::crlfile ""
    if {[info exists extcert(2.5.29.31)]} {
	$w.text insert end [mc "CRL Distribution Points"] bold
	$w.text insert end "\n"
	set listcrl [crlpoints [lindex $extcert(2.5.29.31) 1]]
	foreach {crlp} $listcrl {
	    $w.text insert end "\t[mc "Distribution Point"]:\tURI:$crlp\n"  margins1
	}
	set ::crlfile  $listcrl
	unset extcert(2.5.29.31)
    }    
    if {[info exists extcert(id-ce-issuerAltName)]} {
	$w.text insert end [mc "Issuer Alt Name"] bold
	$w.text insert end "\n"
	set listalt [altname [lindex $extcert(id-ce-issuerAltName) 1]]
	foreach {a b} $listalt {
	    $w.text insert end "\t[mc $a]:\tURI:$b\n"  margins1
	}
	unset extcert(id-ce-issuerAltName)
    }
    if {[info exists extcert(id-ce-subjectAltName)]} {
	$w.text insert end [mc "Subject Alt Name"] bold
	$w.text insert end "\n"
	set listalt [altname [lindex $extcert(id-ce-subjectAltName) 1]]
	foreach {a b} $listalt {
	    $w.text insert end "\t[mc $a]:\tURI:$b\n"  margins1
	}
	unset extcert(id-ce-subjectAltName)
    }
    
    set listext [array get extcert]
    foreach {a b} $listext {
	$w.text insert end [mc "Extension"] bold
	$w.text insert end "\n"
	$w.text insert end "\t[mc "Identifier"]:\t$a\n"  margins1
	$w.text insert end "\t[mc "Value"]:\t[edithex [lindex $b 1]]\n"  margins1
	if {[lindex $b 0] == 1} {
	    set critcert [mc "Yes"]
	} else {
	    set critcert [mc "No"]
	}
	$w.text insert end "\t[mc "Critical"]:\t$critcert\n"  margins1
    }

    $w.text insert end [mc "Signature"] bold
    $w.text insert end "\n"
    $w.text insert end "\t[mc "Signature Algorithm"]\t[mc "$cert_parse(signature_algo)"]\n"  margins1
    $w.text insert end "\t[mc "Signature"]:\t[edithex $cert_parse(signature)]\n"  margins11

    if {$avecdet == 0} {
	typeview $w
    }
}

proc exportparsecert {cert w} {
    global loadfile
    if {$cert == ""} {
	if {$loadfile == ""} {
	    tk_messageBox -title [mc "Save certificate"] -icon info -message [mc "No certificate selected"]
	    return
	}
	set cert [$w.text get 1.0 end]
    }
    global loadfile
    global home
    set dir [tk_chooseDirectory -initialdir $home]
    if {$dir == ""} {
	return
    }
#Сохраняем корневой сертификат в указанном каталоге
    set f2 [file rootname $loadfile]
    set f1 [file tail $f2]
    set f [file join $dir $f1]
    set savefile "$f.txt"
    if {[catch {set fd [open "$savefile" w]} rc]} {
	tk_messageBox -title [mc "Save certificate"] -icon error -message "$savefile" -detail "Файл с разобранным сертификатом"
	return
    }
    puts -nonewline $fd $cert
    close $fd
    tk_messageBox -title [mc "Save certificate"] -icon info -message "$savefile" -detail "Файл с разобранным сертификатом"
    return
}

proc showContextMenu {w x y rootx rooty} {
    global loadfile
    catch {destroy .contextMenu}
    menu .contextMenu -tearoff false
    if {$loadfile != ""} {
	set a [$w get 1.0 end]
    } else {
	set a ""
    }
    .contextMenu add command -label [mc "Save certificate"] -command [list exportparsecert $a $w]
    .contextMenu configure -title "Certificate"
    .contextMenu add command -label [mc "Load chain CA"] \
            -command {::loadchain}
    .contextMenu add command -label [mc "Load CRL"] \
        -command {::loadcrl}
    .contextMenu add separator
    .contextMenu add command -label [mc "About"] -command {aboutUtil .about}

    tk_popup .contextMenu $rootx $rooty
    .contextMenu configure -activebackground #39b5da
    .contextMenu configure -background #e0e0da
}


#######################
frame $w.tit -bg #ffffff
frame $w.tit.left -bg #ffffff -pady 0
frame $w.tit.right -bg #ffffff

option add *Label.background #ffffff
option add *Label.highlightbackground #ffffff
label $w.tit.left.l1 -text "CN владельца сертификата" -anchor sw -font TkFontForFirst
set ident [mc "Identity"]
set ident_name [mc "Subject Name"]
label $w.tit.left.l2 -text "$ident: $ident_name" -relief flat -anchor sw -font TkDefaultFont
set verca [mc "Verified by"]
set verca_name [mc "Issuer Name"]
label $w.tit.left.l3 -text "$verca: $verca_name" -relief flat -anchor sw -font TkDefaultFont
set expir [mc "Expires"]

label $w.tit.left.l4 -text "$expir: Дата" -relief flat -anchor sw -font TkDefaultFont
ttk::style configure Me.TButton -background #ffffff
ttk::style configure Me.TButton  -bd 0  -anchor sw  -shiftrelief 0 -style Me.TButton  -font TkDefaultFontBold \
     -relief flat -bg #ffffff -highlightbackground #ffffff -activebackground #ffffff
ttk::style map Me.TButton -background [list disabled #ffffff pressed #ffffff  active #ffffff] -foreground [list disabled #a3a3a3] -relief [list {pressed !disabled} flat]
ttk::button $w.tit.left.caption -text [mc "Details"] -image ::img::not_det  -compound left  -style Me.TButton  -command {typeview $w}

pack $w.tit.left.l1 -side top -expand yes -pady 1 -anchor w -padx 5
pack $w.tit.left.l2 $w.tit.left.l3 $w.tit.left.l4 -side top -expand yes -pady 0 -anchor w -padx 15
pack $w.tit.left.caption -side top -expand yes -pady 0 -anchor w -padx 5 

label $w.tit.right.picture -relief flat -anchor ne -image ::img::logo -bg #ffffff -compound top
pack $w.tit.right.picture -side top -padx 0
pack $w.tit -side top -fill both -expand 1
pack $w.tit.left  -side left -expand 1 -padx 0 -pady 12 -fill both 
pack $w.tit.right -side right -expand 0 -padx 10 -pady 15 -fill y 
button  $w.tit.right.lang -command {changelang $w} -image ::img::us_32x32 -compound left -bd 0 -background #ffffff -activebackground #ffffff -highlightthickness 0
pack $w.tit.right.lang -side bottom

text $w.text -yscrollcommand [list $w.scroll set] -setgrid 1  -relief flat -font TkDefaultFont -exportselection 1 -wrap word \
	-height 20 -undo 1 -autosep 1 -bg #ffffff  -highlightbackground #ffffff	 -highlightcolor #ffffff 

switch $typesys {
    win32        { 
ttk::scrollbar $w.scroll -command [list $w.text yview] 
    }
    x11 { 
	ttk::style map My.Vertical.TScrollbar -troughcolor [list disabled wheat pressed #a3a3a3  active #c3c3c3] -foreground [list disabled #a3a3a3] -relief [list {pressed !disabled} sunken] -background  [list disabled #93cee9 pressed #93cee9  active #93cee9] 
	ttk::style map My.Vertival.TScrollbar -arrowcolor [list disabled #ffffff pressed #ffffff  active #ffffff ]
	ttk::style configure My.Vertical.TScrollbar -troughcolor {#eff0f1} 
	ttk::style configure My.Vertical.TScrollbar -arrowcolor {#eff0f1}
	ttk::style configure My.Vertical.TScrollbar -background #898989 -width 0 -relief flat -borderwidth 0 -arrowsize 5 -arrowcolor #ffffff
	ttk::scrollbar $w.scroll -command [list $w.text yview] -orient vertical -style My.Vertical.TScrollbar
    }
    classic - aqua { 
	ttk::scrollbar $w.scroll -command [list $w.text yview] 
    }
}


bind $w.text <ButtonPress-3> {showContextMenu %W %x %y %X %Y}

$w.text configure -foreground #ffffff
pack $w.scroll -anchor center -expand 0 -fill y -side right  -padx {0 5} -pady {2 8}

pack $w.text -expand yes -fill both -side bottom
pack forget $w.scroll
    frame $w.sep -height 1 -relief flat -bg #b6b6b3
pack $w.sep -side bottom -fill x -expand 1 -pady 0
$w.text tag configure bold -font TkDefaultFontBold
$w.text tag configure super -offset 4p -font TkDefaultFont
$w.text tag configure sub -offset -2p -font TkDefaultFont
$w.text tag configure margins -lmargin1 4m -lmargin2 12m -rmargin 10m
$w.text tag configure margins1 -lmargin1 2m -lmargin2 2.2i -rmargin 2m -spacing1 1p -spacing2 1p -spacing3 1p
$w.text tag configure margins11 -lmargin1 2m -lmargin2 2.2i -rmargin 2m -spacing1 1p -spacing2 1p -spacing3 1p -font TkFixedFont
$w.text tag configure margins2 -lmargin1 6m -lmargin2 2.2i -rmargin 2m -spacing1 1p -spacing2 1p -spacing3 1p
$w.text tag configure spacing -spacing1 10p -spacing2 2p \
	-lmargin1 12m -lmargin2 6m -rmargin 10m
$w.text delete 0.0 end
$w.text configure -tabs {1m 2.2i}
if {$lang == "ru"} {
    $w.tit.right.lang configure -image  ::img::ru_32x32 -compound left
} else {
    $w.tit.right.lang configure -image  ::img::us_32x32 -compound left
}
if {$loadfile != ""} {
    fileWithCert $w $loadfile
}
typeview $w

